# Copyright (c) Art. Lebedev | http://www.artlebedev.ru/
# Updated 2012-09-28 by vlalek

@CLASS
oauth

@init[params]
^if($params is hash){
	^params.foreach[k;v]{
		$self.$k[$v]
	}
}
###


@go_to_auth[]
$response:refresh[ 
	$.value(0) 
	$.url[$self.auth_url] 
]
###


@SET_auth_url[url]
$self._auth_url[$url]
@GET_auth_url[]
$result[$self._auth_url]
###


@SET_token_url[url]
$self._token_url[$url]
@GET_token_url[]
$result[$self._token_url]
###


@SET_data_url[url]
$self._data_url[$url]
@GET_data_url[]
$result[$self._data_url]
###


@_parse_string_result[s]
$result[^s.match[(.*?)=(.*?)(&|^$);g]]
$result[^result.hash[1;2;$.type[string]]]
###


@_parse_json_result[s]
$result[^json:parse[^untaint{$s}]]
###


@assume[data]
^if($data is table){
	$data[$data.fields]
}
^if($self.data is hash && $data is hash){
	^self.data.add[$data]
}{
	$self.data[$data]
}
$result[]
###


@get[name]
$result[^switch[$name]{
	^case[]{$self.data}
	^case[type]{^self.CLASS_NAME.mid(6)}
	^case[DEFAULT]{$self.data.$name}
}]
###


# CLASS 
@CLASS
oauth1

@BASE
oauth

@OPTIONS
locals

@auto[]
$self.now[^date::now[]]
###


@login[]
$params[
	$.oauth_callback[$self.redirect_uri]
	$.oauth_consumer_key[$self.consumer_key]
	$.oauth_nonce[$s[^math:uid64[]^math:uid64[]]^s.lower[]]
	$.oauth_signature_method[HMAC-SHA1]
	$.oauth_timestamp[^self.now.unix-timestamp[]]
	$.oauth_token[]
	$.oauth_version[1.0]

	$.oauth_consumer_secret[$self.consumer_secret]
]

^if(!def $form:oauth_token){
	^self.init[^self.load[
		$params
		$.url[$self.token_url]
		$.method[POST]
	]]
	^self.go_to_auth[]
}{
	^self.assume[^self.load[
		$params
		$.url[$self.access_token_url]
		$.method[POST]
		$.oauth_token[$form:oauth_token]
	]]
	^self.assume[^self.load[
		$params
		$.url[$self.data_url]
		$.method[GET]
		$.format[json]
		$.oauth_callback[]
		$.oauth_token[^self.get[oauth_token]]
		$.oauth_token_secret[^self.get[oauth_token_secret]]
	]]
}
###


@GET_auth_url[]
$result[^BASE:GET_auth_url[]]
$result[$result?oauth_token=$self.oauth_token]
###


@load[params]
$oauth[
	$.oauth_callback[$params.oauth_callback]
	$.oauth_consumer_key[$params.oauth_consumer_key]
	$.oauth_nonce[$params.oauth_nonce]
	$.oauth_signature[]
	$.oauth_signature_method[$params.oauth_signature_method]
	$.oauth_timestamp[$params.oauth_timestamp]
	$.oauth_token[$params.oauth_token]
	$.oauth_version[$params.oauth_version]
]


$signature_base_string[$params.method&^self.taint_uri[$params.url]&]
$i(0)
^oauth.foreach[k;v]{
	^if(def $v){
		$signature_base_string[$signature_base_string^self.taint_uri[^if($i){&}$k=^self.taint_uri[$v]]]
		^i.inc[]
	}
}

$composite_key[^self.taint_uri[$params.oauth_consumer_secret]&^self.taint_uri[$params.oauth_token_secret]]
$oauth.oauth_signature[^math:digest[sha1;$signature_base_string;$.format[base64]$.hmac[$composite_key]]]

$authorization_header[OAuth ]
$i(0)
^oauth.foreach[k;v]{
	^if(def $v){
		$authorization_header[$authorization_header^if($i){,} $k="^self.taint_uri[$v]"]
		^i.inc[]
	}
}

$f[^curl:load[
	^if($params.method eq POST){$.post(true)}
	$.url[${params.url}$params.url_params]
	$.timeout(10)
	$.ssl_verifypeer(false)
	$.httpheader[
		$.Authorization[$authorization_header]
	]
]]
$result[^switch[$params.format]{
	^case[json]{^self._parse_json_result[$f.text]}
	^case[DEFAULT]{^self._parse_string_result[$f.text]}
}]
###


@taint_uri[text]
$result[^apply-taint[^taint[uri][$text]]]
$result[^result.match[/;g;%2F]]
###



@CLASS
oauth_twitter

@BASE
oauth1

@OPTIONS
locals

@init[params]
$self.auth_url[https://api.twitter.com/oauth/authenticate]
$self.token_url[https://api.twitter.com/oauth/request_token]
$self.access_token_url[https://api.twitter.com/oauth/access_token]
$self.data_url[https://api.twitter.com/1/account/verify_credentials.json]

^BASE:init[$params]
###


@get[name]
$result[^switch[$name]{
	^case[username]{$self.data.screen_name}
	^case[DEFAULT]{^BASE:get[$name]}
}]
###


@CLASS
oauth2

@BASE
oauth

@OPTIONS
locals


@login[]
^if(!def $form:code){
	^self.go_to_auth[]
}{
	^self.load_token[]
	^self.load_data[]
}
###



@GET_auth_url[]
$result[^BASE:GET_auth_url[]]
$result[$result?redirect_uri=^taint[uri][$self.redirect_uri]]
$result[$result&client_id=$self.client_id]
###


@load_token[]
$f[^curl:load[
	$.post(true)
	$.url[$self.token_url]
	$.timeout(10)
	$.ssl_verifypeer(false)
]]
^self.init[^self.parse_token_result[$f.text]]
$result[]
###


@parse_token_result[s]
$result[^self._parse_json_result[$s]]
###


@GET_token_url[]
$result[^BASE:GET_token_url[]]
$result[$result?redirect_uri=^taint[uri][$self.redirect_uri]]
$result[$result&client_id=$self.client_id]
$result[$result&client_secret=$self.client_secret]
$result[$result&code=$form:code]
###


@load_data[]
$f[^curl:load[
	$.url[$self.data_url]
	$.timeout(10)
	$.ssl_verifypeer(false)
]]
$self.data[^self.parse_data_result[$f.text]]
$result[]
###


@parse_data_result[s]
$result[^self._parse_json_result[$s]]
###


@GET_data_url[]
$result[^BASE:GET_data_url[]]
$result[$result?access_token=$self.access_token]
###



@CLASS
oauth_facebook

@BASE
oauth2

@OPTIONS
locals

@init[params]
$self.auth_url[http://www.facebook.com/dialog/oauth]
$self.token_url[https://graph.facebook.com/oauth/access_token]
$self.data_url[https://graph.facebook.com/me]

^BASE:init[$params]
###


@GET_auth_url[]
$result[^BASE:GET_auth_url[]]
$result[$result&scope=email]
#$result[$result&state=$cookie:_SESSION_STATE]
###


@parse_token_result[s]
$result[^self._parse_string_result[$s]]
###



@CLASS
oauth_vk

@BASE
oauth2

@OPTIONS
locals

@init[params]
$self.auth_url[http://oauth.vk.com/authorize]
$self.token_url[https://oauth.vk.com/access_token]
$self.data_url[https://api.vk.com/method/getProfiles]

^BASE:init[$params]
###


@get[name]
$result[^switch[$name]{
	^case[id]{^BASE:get[uid]}
	^case[username]{^BASE:get[screen_name]}
	^case[name]{^BASE:get[first_name] ^BASE:get[last_name]}
	^case[gender]{^switch[^BASE:get[sex]]{^case[1]{female}^case[2]{male}}}
	^case[birthday]{$s[^BASE:get[bdate]]^s.match[^^.*?(\d\d?).(\d\d?)\.(\d{4}).*^$;]{${match.3}-${match.2}-${match.1}}}
	^case[email]{$self.email}
	^case[DEFAULT]{^BASE:get[$name]}
}]
###


@GET_auth_url[]
$result[^BASE:GET_auth_url[]]
$result[$result&scope=email]
$result[$result&response_type=code]
###


@parse_data_result[s]
$result[^self._parse_json_result[$s]]
$result[$result.response.0]
###


@GET_data_url[]
$result[^BASE:GET_data_url[]]
$result[$result&uid=$self.user_id]
$result[$result&fields=uid,email,first_name,last_name,nickname,screen_name,sex,birthdate,city,country,timezone,photo,photo_medium,photo_big,has_mobile,rate,contacts,education,online,counters]
###



@CLASS
oauth_odnoklassniki

@BASE
oauth2

@OPTIONS
locals

@init[params]
$self.auth_url[http://ok.ru/oauth/authorize]
$self.token_url[http://api.ok.ru/oauth/token.do]
$self.data_url[http://api.ok.ru/fb.do]

^BASE:init[$params]
###


@get[name]
$result[^switch[$name]{
	^case[id]{^BASE:get[uid]}
	^case[username]{^BASE:get[name]}
	^case[DEFAULT]{^BASE:get[$name]}
}]
###


@GET_auth_url[]
$result[^BASE:GET_auth_url[]]
$result[$result&scope=VALUABLE_ACCESS&response_type=code]
###


@GET_token_url[]
$result[^BASE:GET_token_url[]]
$result[$result&grant_type=authorization_code]
###


@GET_data_url[]
$result[^BASE:GET_data_url[]]
$result[$result&method=users.getCurrentUser]
$result[$result&application_key=$self.application_key]
$result[$result&sig=^math:md5[application_key=${self.application_key}method=users.getCurrentUser^math:md5[${self.access_token}$self.client_secret]]]
###
